@@ -58,7 +58,7 @@ And to get back signal data::
   repository = Repository("http://demo.biosignalml.org")
 
   # Retrieve the recording and all metadata about its signals:
-  recording = repository.get_recording_with_signals("http//example.org/recording/demo/1")
+  recording = repository.get_recording("http//example.org/recording/demo/1")
 
   # Select a signal by its signal identifier (can also use a URI):
   signal = recording.signal(id="id2")
@@ -87,7 +87,7 @@ import biosignalml.rdf as rdf
 from biosignalml            import BSML
 from biosignalml.data       import TimeSeries, UniformTimeSeries, DataSegment
 from biosignalml.data.time  import Interval
-from biosignalml.formats    import BSMLRecording, BSMLSignal
+from biosignalml.formats    import BSMLRecording, BSMLSignal, MIMETYPES
 from biosignalml.transports import StreamException
 from biosignalml.transports import BlockType, SignalData
 from biosignalml.transports import WebStreamReader, WebStreamWriter, StreamException
@@ -95,66 +95,27 @@ from biosignalml.transports import WebStreamReader, WebStreamWriter, StreamExcep
 from . import repository
 
 
-def _get_token_file():
-#=====================
-  path = os.getenv('HOME') + '/.bsml'
-  try: os.makedirs(path)
-  except OSError: pass
-  try:
-    return open(path + '/' + 'tokens', 'r+')
-  except IOError:
-    return open(path + '/' + 'tokens', 'w+')
-
-def save_token(repository, token):
-#=================================
-  f = _get_token_file()
-  g = tempfile.NamedTemporaryFile(delete=True)
-  existing = False
-  for l in f:
-    if repository == l.split()[0]:
-      existing = True
-      g.write('%s %s\n' % (repository, token))
-    else:
-      g.write(l)
-  if not existing: g.write('%s %s\n' % (repository, token))
-  g.flush()
-  f.seek(0)
-  f.truncate()
-  g.seek(0)
-  for l in g: f.write(l)
-  f.close()
-  g.close()
-
-
-def get_token(repository):
-#=========================
-  token = None
-  f = _get_token_file()
-  for l in f:
-    p = l.split()
-    if repository == p[0] and len(p) > 1:
-      token = p[1]
-      break
-  f.close()
-  return token
-
-
 class Signal(BSMLSignal):
 #========================
 
   def __init__(self, uri, units=None, **kwds):
   #-------------------------------------------
     repository = kwds.pop('repository', None)
+    self._length = 0
     super(Signal, self).__init__(uri, units, **kwds)
     self.repository = repository
 
   def close(self):
   #---------------
-    # Ensure all metadata has been POSTed
+    # Ensure all metadata has been POSTed (happens when recording is closed)
     pass
 
-  def read(self, interval=None, segment=None, maxpoints=0, dtype=None, units=None):
-  #--------------------------------------------------------------------------------
+  def __len__(self):
+  #----------------
+    return self._length
+
+  def read(self, interval=None, segment=None, maxpoints=0, dtype=None, rate=None, units=None):
+  #-------------------------------------------------------------------------------------------
     params = { }
     if interval:
       if isinstance(interval, Interval):
@@ -168,6 +129,7 @@ class Signal(BSMLSignal):
       params['count'] = segment[1]
     if maxpoints: params['maxsize'] = maxpoints
     if dtype is not None: params['dtype'] = dtype
+    if rate is not None: params['rate'] = rate
     if units is not None: params['units'] = units
     for sd in self.repository.get_data(str(self.uri), **params):
       if sd.uri != str(self.uri):
@@ -177,6 +139,7 @@ class Signal(BSMLSignal):
 
   def append(self, timeseries):
   #----------------------------
+    self._length += len(timeseries)
     if isinstance(timeseries, TimeSeries):
       return self.repository.put_data(str(self.uri), timeseries)
     elif self.rate:
@@ -197,7 +160,12 @@ class Recording(BSMLRecording):
   def close(self):
   #---------------
     # Ensure all metadata has been POSTed
-    pass
+    self.repository.post_metadata(self.uri, self.metadata_as_string())
+#    for sig in self.signals():
+#      self.repository.post_metadata(sig.uri, sig.metadata_as_string())
+
+    ## POST duplicates duration but leaves dataset...
+    ## PUT removes dataset,,,
 
   def add_signal(self, signal):
   #----------------------------
@@ -211,9 +179,6 @@ class Recording(BSMLRecording):
     # it will create an signal group in HDF5 container
     try:
       sig = super(Recording, self).new_signal(uri, units, id=id, repository=self.repository, **kwds)
-      ## This should spot duplicates, even if we have done get_recording()
-      ## and not get_recording_with_signals()
-
       #logging.debug('New Signal: %s --> %s', sig.uri, sig.attributes)
       self.repository.post_metadata(self.uri, sig.metadata_as_string())
       return sig
@@ -221,8 +186,9 @@ class Recording(BSMLRecording):
       raise
       raise IOError("Cannot create Signal '%s' in repository" % uri)
 
-  def save_metadata(self, metadata, format=rdf.Format.RDFXML):
-  #-----------------------------------------------------------
+  def save_metadata(self, metadata=None, format=rdf.Format.RDFXML):
+  #----------------------------------------------------------------
+    if metadata is None: metadata = self.metadata_as_string()
     self.repository.post_metadata(self.uri, metadata, format)
 
 
@@ -230,19 +196,19 @@ class Recording(BSMLRecording):
 class Repository(repository.RemoteRepository):
 #=============================================
 
+  ## Can a Repository by stateless, created on-the-fly as needed??
+  ## Then get/new methods would be static
+
   RecordingClass = Recording      #: The class of recordings in the repository.
 
   def __init__(self, uri, **kwds):
   #-------------------------------
-    super(Repository, self).__init__(uri, access_key=get_token(uri), **kwds)
-
-  @classmethod
-  def connect(cls, uri, **kwds):
-  #-----------------------------
     p = urlparse.urlparse(uri)
-    if '' in [p.scheme, p.netloc]:
+    if p.scheme == '' or p.hostname is None:
       raise IOError("Invalid URI -- %s" % uri)
-    return cls(p.scheme + '://' + p.netloc, **kwds)
+    uri = p.scheme + '://' + p.hostname
+    kwds['port'] = p.port
+    super(Repository, self).__init__(uri, access_key=Repository.get_token(uri), **kwds)
 
   def get_recording(self, uri, graph_uri=None, **kwds):
   #----------------------------------------------------
@@ -260,6 +226,7 @@ class Repository(repository.RemoteRepository):
       rec = self.RecordingClass(uri, repository=self, **kwds)
       # Will have default metadata with attributes set from any metadata keyword dict
       rec.graph = rdf.Graph(self.put_metadata(rec.uri, rec.metadata_as_string()))
+      rec.format = MIMETYPES.HDF5
       # Format = HDF5 (BSML ??)
       # then when server processes PUT for a new BSML recording it will create an empty HDF5 container
       return rec
@@ -292,6 +259,8 @@ class Repository(repository.RemoteRepository):
     offset
     count
     dtype
+    rate
+    units
     '''
     kwds['access_key'] = self._access_key
     reader = WebStreamReader(self._web_sockets_uri(uri), uri, **kwds)
@@ -304,8 +273,8 @@ class Repository(repository.RemoteRepository):
     stream = None
     try:
       stream = WebStreamWriter(self._web_sockets_uri(uri), access_key=self._access_key)
-      MAXPOINTS = 50000   ##### TESTING    (200K bytes if double precision)
-      params = { }
+      MAXPOINTS = 10000
+      params = { 'dtype': 'f4' }
       if hasattr(timeseries, 'rate'): params['rate'] = timeseries.rate
       pos = 0
       count = len(timeseries)
@@ -322,6 +291,53 @@ class Repository(repository.RemoteRepository):
     finally:
       if stream: stream.close()
 
+  @staticmethod
+  def _get_token_file():
+  #---------------------
+    path = os.getenv('HOME') + '/.bsml'
+    try: os.makedirs(path)
+    except OSError: pass
+    try:
+      return open(path + '/' + 'tokens', 'r+')
+    except IOError:
+      return open(path + '/' + 'tokens', 'w+')
+
+  @staticmethod
+  def get_token(uri):
+  #------------------
+    token = None
+    f = Repository._get_token_file()
+    for l in f:
+      p = l.split()
+      if str(uri) == p[0] and len(p) > 1:
+        token = p[1]
+        break
+    f.close()
+    return token
+
+  @staticmethod
+  def save_token(uri, token):
+  #--------------------------
+    f = Repository._get_token_file()
+    g = tempfile.NamedTemporaryFile(delete=True)
+    existing = False
+    uri = str(uri)
+    for l in f:
+      if uri == l.split()[0]:
+        existing = True
+        g.write('%s %s\n' % (uri, token))
+      else:
+        g.write(l)
+    if not existing:
+      g.write('%s %s\n' % (uri, token))
+    g.flush()
+    f.seek(0)
+    f.truncate()
+    g.seek(0)
+    for l in g: f.write(l)
+    f.close()
+    g.close()
+
 
 if __name__ == "__main__":
 #=========================
@@ -333,7 +349,7 @@ if __name__ == "__main__":
   rec_uri = 'http://devel.biosignalml.org/fph/icon/120312170352/FLW0002'
   sig_uri = rec_uri + '/signal/2'
 
-  rec = repo.get_recording_with_signals(rec_uri)
+  rec = repo.get_recording(rec_uri)
   for s in sorted(rec.signals(), key=lambda s: str(s.uri)):
     print s.uri, s.label
 #    for d in s.read(): print d
