@@ -60,7 +60,7 @@ class Signal(core.AbstractObject):
 
   attributes = ['recording', 'units', 'transducer', 'filter', '_rate',  '_period', 'clock',
                 'minFrequency', 'maxFrequency', 'minValue', 'maxValue',
-                'index', 'signaltype', 'offset', 'duration',
+                'index', 'signaltype', 'offset', '_duration',
                ]
   '''Generic attributes of a Signal.'''
 
@@ -81,7 +81,7 @@ class Signal(core.AbstractObject):
               'offset':       PropertyMap(BSML.offset, XSD.dayTimeDuration,
                                           utils.seconds_to_isoduration,
                                           utils.isoduration_to_seconds),
-              'duration':     PropertyMap(DCT.extent, XSD.dayTimeDuration,
+              '_duration':    PropertyMap(DCT.extent, XSD.dayTimeDuration,
                                           utils.seconds_to_isoduration,
                                           utils.isoduration_to_seconds),
             }
@@ -96,6 +96,14 @@ class Signal(core.AbstractObject):
     kwds['_period'] = period
     core.AbstractObject.__init__(self, uri, units=units, **kwds)
 
+  def __len__(self):
+  #----------------
+    return 0
+
+  def __nonzero__(self):
+  #---------------------
+    return True  # Otherwise bool(sig) is False, because we have __len__()
+
   @property
   def rate(self):
   #==============
@@ -108,6 +116,13 @@ class Signal(core.AbstractObject):
     if   self._period is not None: return float(self._period)
     elif self._rate   is not None: return 1.0/float(self._rate)
 
+  @property
+  def duration(self):
+  #------------------
+    if self._duration is not None: return self._duration
+    elif self.period is not None: return len(self)*self.period
+    ## self.time[-1] + (self.period if self.period is not None else 0)  ???
+
 
 class Event(core.AbstractObject):
 #================================
@@ -363,26 +378,22 @@ class Annotation(core.AbstractObject):
   '''
   metaclass = BSML.Annotation  #: :attr:`.BSML.Annotation`
 
-  attributes = [ 'about', 'comment', 'tags', 'time', 'creator', 'created' ]
+  attributes = [ 'about', 'comment', 'tags', 'time' ]
 
   mapping = { 'about':   PropertyMap(DCT.subject, to_rdf=mapping.get_uri),
               'comment': PropertyMap(RDFS.comment),
               'tags':    PropertyMap(BSML.tag),
               'time':    PropertyMap(BSML.time, subelement=True),
-              'creator': PropertyMap(DCT.creator, to_rdf=mapping.get_uri),
-              'created': PropertyMap(DCT.created, XSD.dateTime,
-                                     utils.datetime_to_isoformat,
-                                     utils.isoformat_to_datetime),
             }
 
 
-  def __init__(self, uri, about=None, comment=None, tags=None, time=None, creator=None, timestamp=True, **kwds):
-  #-------------------------------------------------------------------------------------------------------------
+  def __init__(self, uri, about=None, comment=None, tags=None, time=None, timestamp=True, **kwds):
+  #-----------------------------------------------------------------------------------------------
     if time is not None: assert(time.end >= time.start)   ###
     created = kwds.pop('created', utils.utctime()) if timestamp else None
     label = kwds.get('label', '')
     core.AbstractObject.__init__(self, uri, about=about, comment=comment, time=time,
-      creator=creator, created=created, **kwds)
+      created=created, **kwds)
     self.tags = tags if tags else []
 
   @classmethod
