@@ -28,6 +28,7 @@ available from http://librdf.org/.
 
 import sys
 import uuid
+import atexit
 import logging
 
 import RDF as librdf
@@ -102,6 +103,13 @@ NAMESPACES = {
 for prefix, name in NAMESPACES.iteritems():
   setattr(sys.modules[__name__], prefix.upper(), NS(name))
 
+#def _clean_prefixes():
+##---------------------
+#  for prefix in NAMESPACES:
+#    del getattr(sys.modules[__name__], prefix.upper())
+#
+#atexit.register(_clean_prefixes)
+
 
 class Uri(librdf.Uri):
 #=====================
@@ -173,7 +181,7 @@ class Literal(Node):
   #-------------------------------------------------------
     super(Literal, self).__init__(literal=str(value), datatype=Uri(datatype), language=language)
 
-  def as_string(self):
+  def as_string(self):   ## V's __str__ ???
   #-------------------
     '''
     Return the literal as a quoted string with language and datatype attributes.
@@ -194,6 +202,8 @@ class BlankNode(Node):
   '''
   def __init__(self, blank=None):
   #------------------------------
+    if blank is not None and blank.startswith('nodeID://'):
+      blank = str(blank[9:])    ## Tidy Virtuoso blank node identifiers
     super(BlankNode, self).__init__(blank=blank)
 
   def __str__(self):
@@ -370,8 +380,8 @@ class Graph(librdf.Model):
     self.uri = uri
 
   @classmethod
-  def create_from_resource(cls, uri, format, base=None):
-  #-----------------------------------------------------
+  def create_from_resource(cls, uri, format=Format.RDFXML, base=None):
+  #-------------------------------------------------------------------
     """
     Create a new Graph from RDF statements in a resource.
 
@@ -385,8 +395,8 @@ class Graph(librdf.Model):
     return self
 
   @classmethod
-  def create_from_string(cls, uri, string, format):
-  #------------------------------------------------
+  def create_from_string(cls, uri, string, format=Format.RDFXML):
+  #--------------------------------------------------------------
     """
     Create a new Graph from RDF statements in a string.
 
@@ -405,8 +415,8 @@ class Graph(librdf.Model):
   #-----------------
     return str(self.uri)
 
-  def parse_resource(self, uri, format, base=None):
-  #------------------------------------------------
+  def parse_resource(self, uri, format=Format.RDFXML, base=None):
+  #--------------------------------------------------------------
     """
     Add statements to the graph from a resource.
 
@@ -422,8 +432,8 @@ class Graph(librdf.Model):
     except librdf.RedlandError, msg:
       raise RDFParseError, msg
 
-  def parse_string(self, string, format, base):
-  #--------------------------------------------
+  def parse_string(self, string, format=Format.RDFXML, base=None):
+  #---------------------------------------------------------------
     """
     Add statements to the graph from a string.
 
@@ -480,7 +490,8 @@ class Graph(librdf.Model):
 
     :param graph: A :class:`Graph` containing statements.
     '''
-    for s in graph: self.append(s)
+    if graph is not None:
+      for s in graph: self.append(s)
 
   def contains(self, statement):
   #-----------------------------
@@ -593,7 +604,7 @@ class Graph(librdf.Model):
     Append the statement (s, p, o) to the graph after first removing
     all statements with subject `s` and predicate `p`.
     """
-    for stmt in self.find_statements(Statement(s, s, None)):
+    for stmt in self.find_statements(Statement(s, p, None)):
       del self[stmt]
     self.append(Statement(s, p, o))
 
