@@ -73,22 +73,31 @@ class SparqlStore(object):
 
   """
 
-  def __init__(self, href):
-  #------------------------
+  def __init__(self, href, port=None):
+  #-----------------------------------
     self._href = href
+    self._port = port
 
   def _request(self, endpoint, method, body=None, headers=None):
   #-------------------------------------------------------------
-    endpoint = self._href + endpoint
+    if self._port is None:
+      endpoint = self._href + endpoint
+    else:
+      endpoint = self._href + ':' + str(self._port) + endpoint
     try:
       http = httplib2.Http(timeout=20)
       response, content = http.request(endpoint, body=body, method=method, headers=headers)
     except socket.error, msg:
       raise StoreException("Cannot connect to SPARQL endpoint: %s" % endpoint)
     if response.status not in [200, 201]:
-      raise StoreException('SPARQL request HTTP error %d: %s' % (response.status, response.reason))
+      raise StoreException('SPARQL error: %s' % self._error_text(response, content))
     return content
 
+  @staticmethod
+  def _error_text(response, content):
+  #----------------------------------
+    return response.reason
+
   def http_request(self, endpoint, method, body=None, headers=None):
   #-----------------------------------------------------------------
     return self._request(endpoint, method, body, headers)
@@ -110,7 +119,7 @@ class SparqlStore(object):
                                headers={'Content-type': 'application/x-www-form-urlencoded',
                                         'Accept': rdf.Format.mimetype(format)} )
     except Exception, msg:
-      logging.error('SPARQL: %s, %s', msg, self.map_prefixes(prefixes) + sparql)
+      logging.debug('SPARQL: %s, %s', msg, self.map_prefixes(prefixes) + sparql)
       raise
 
   def ask(self, where, params=None, graph=None, prefixes=None):
@@ -147,10 +156,10 @@ class SparqlStore(object):
     '''
     if params is None: params = {}
     return json.loads(self.query(
-              'select%(distinct)s %(fields)s where {\n %(graph)s {\n %(where)s }\n }%(group)s%(order)s%(limit)s'
+              'select%(distinct)s %(fields)s %(graph)s where {\n %(where)s\n}%(group)s%(order)s%(limit)s'
                  % dict(distinct=' distinct' if distinct else '',
                         fields=fields % params,
-                        graph=('graph <%s>' % graph) if graph else '',
+                        graph=('from <%s>' % graph) if graph else '',
                         where=where % params,
                         group=(' group by %s' % group) if group else '',
                         order=(' order by %s' % order) if order else '',
@@ -158,8 +167,9 @@ class SparqlStore(object):
                  rdf.Format.JSON, prefixes)
         ).get('results', {}).get('bindings', [])
 
-  def construct(self, template, where, params=None, graph=None, format=rdf.Format.RDFXML, prefixes=None):
-  #------------------------------------------------------------------------------------------------------
+  def construct(self, template, where=None, params=None, graph=None, format=rdf.Format.RDFXML, prefixes=None):
+  #-----------------------------------------------------------------------------------------------------------
+    if where is None: where = template
     if params is None: params = {}
     return self.query('construct { %(tplate)s } %(graph)s where { %(where)s }'
                       % dict(tplate=template % params,
@@ -201,7 +211,7 @@ class SparqlUpdateStore(SparqlStore):
                                body=urllib.urlencode({self.UPDATE_PARAMETER: sparql + self.map_prefixes(prefixes)}),
                                headers={'Content-type': 'application/x-www-form-urlencoded'})
     except Exception, msg:
-      logging.error('SPARQL: %s, %s', msg, sparql + self.map_prefixes(prefixes))
+      logging.debug('SPARQL: %s, %s', msg, sparql + self.map_prefixes(prefixes))
       raise
 
 
@@ -216,7 +226,7 @@ class SparqlUpdateStore(SparqlStore):
                 % { 'graph': str(graph),
                     'triples': ' . '.join([' '.join(list(s)) for s in triples ]) })
     content = self.update(sparql, prefixes)
-    if 'error' in content: raise Exception(content)
+    if 'error' in content: raise StoreException(content)
 
   def delete_triples(self, graph, triples, prefixes=None):
   #------------------------\-------------------------------
@@ -229,7 +239,21 @@ class SparqlUpdateStore(SparqlStore):
                 % { 'graph': graph,
                     'triples': ' . '.join([' '.join(list(s)) for s in triples ]) })
     content = self.update(sparql, prefixes)
-    if 'error' in content: raise Exception(content)
+    if 'error' in content: raise StoreException(content)
+
+## These need separating into SPARQL 1.1 Update and Virtuoso SPARUL methods...
+# DELETE WHERE { GRAPH <%(g)s> { <%(s)s> <%(p)s> ?o } }   ## SPARQL 1.1
+#Sparql 1.1
+#WITH <%(graph)s>
+#  DELETE { <%(subject)s> <%(predicate>s> ?o  }
+#  INSERT { <%(subject)s> <%(predicate>s> %(value)s }
+#  WHERE  { <%(subject)s> <%(predicate>s> ?o }
+#
+#Virtuoso:
+#MODIFY GRAPH <%(graph)s>
+#  DELETE { <%(subject)s> <%(predicate>s> ?o  }
+#  INSERT { <%(subject)s> <%(predicate>s> %(value)s }
+#  FROM <%(graph)s> WHERE  { <%(subject)s> <%(predicate>s> ?o }
 
   def update_triples(self, graph, triples, prefixes=None):
   #-------------------------------------------------------
@@ -246,7 +270,7 @@ class SparqlUpdateStore(SparqlStore):
         sparql = ('delete from graph <%(g)s> { %(s)s %(p)s ?o } from <%(g)s> where { %(s)s %(p)s ?o }'
                     % {'g': str(graph), 's': s, 'p': p} )
         content = self.update(sparql, prefixes)
-        if 'error' in content: raise Exception(content)
+        if 'error' in content: raise StoreException(content)
         last = (s, p)
     self.insert_triples(graph, triples, prefixes)  ###### DUPLICATES BECAUSE OF 4STORE BUG...
 
@@ -301,6 +325,10 @@ class Virtuoso(SparqlUpdateStore):
   UPDATE_PARAMETER = 'query'
   GRAPH_PARAMETER = 'graph-uri'
 
+  @staticmethod
+  def _error_text(response, content):
+  #----------------------------------
+    return content.split('\n')[0].split(':')[-1].strip()
 
   def extend_graph(self, graph, statements, format=rdf.Format.RDFXML):
   #-------------------------------------------------------------------
