@@ -89,6 +89,11 @@ class GraphStore(object):
     self._sparqlstore = sparqlstore
     self._provenance_uri = self.uri + PROVENANCE_PATH
 
+  @property
+  def provenance_uri(self):
+  #------------------------
+    return self._provenance_uri
+
   def has_provenance(self, graph_uri):
   #-----------------------------------
     ''' Check a URI is that of a graph for which we have current provenance.'''
@@ -96,7 +101,7 @@ class GraphStore(object):
       '''graph <%(pgraph)s> { <%(graph)s> a <%(gtype)s> MINUS { [] prv:precededBy <%(graph)s> }}
          graph <%(graph)s> { [] a [] }''',
       params=dict(pgraph=self._provenance_uri, graph=graph_uri, gtype=self._graphtype),
-      prefixes=dict(bsml=BSML.prefix))
+      prefixes=dict(bsml=BSML.prefix, prv=PRV.prefix))
 
 
   def get_provenance(self, graph_uri):
@@ -202,19 +207,20 @@ class GraphStore(object):
       rdf = self._sparqlstore.construct('<%(uri)s> ?p ?o',
               'graph <%(graph)s> { <%(uri)s> a <%(rtype)s> . <%(uri)s> ?p ?o }',
               params=dict(graph=graph_uri, uri=uri, rtype=rtype), format=Format.RDFXML)
-
     ## Virtuoso has a MaxRows limit in its INI file with a default of 10000.
     ## This has been increased to 50000
-    return Graph.create_from_string(graph_uri, rdf, Format.RDFXML)
+    resource = Graph.create_from_string(graph_uri, rdf, Format.RDFXML)
+    return resource if resource.contains(Statement(uri, RDF.type, rtype)) else None
 
 
   def get_resource_graph_uri(self, uri):
   #-------------------------------------
-     for r in self.select('?g',
-       '''graph <%(pgraph)s> { ?g a <%(gtype)s> MINUS { [] prv:precededBy ?g }}
-          graph ?g { <%(uri)s> a [] }''',
-         params=dict(pgraph=self._provenance_uri, gtype=self._graphtype, uri=uri)):
-       return sparqlstore.get_result_value(r, 'g')
+    for r in self.select('?g',
+      '''graph <%(pgraph)s> { ?g a <%(gtype)s> MINUS { [] prv:precededBy ?g }}
+         graph ?g { <%(uri)s> a [] }''',
+        params=dict(pgraph=self._provenance_uri, gtype=self._graphtype, uri=uri),
+        prefixes=dict(prv=PRV.prefix)):
+      return sparqlstore.get_result_value(r, 'g')
 
 
   def query(self, sparql, header=False):
@@ -424,3 +430,22 @@ class GraphUpdate(GraphStore):
     self.extend_graph(self._provenance_uri, prov.metadata_as_graph().serialise())
     self.replace_graph(graph_uri, rdf, format=format)
     return graph_uri
+
+  def save_subject_property(self, graph_uri, s, p):
+  #------------------------------------------------
+    """
+    """
+    o = getattr(s, p, None)
+    ### Following should be done in Mapping class...
+    if o not in ['', None] and isinstance(s, model.core.AbstractObject):
+      mc = [ c.metaclass for c in s.__class__.__mro__ if c.__dict__.get('metaclass') ]
+      for k, m in s.rdfmap.mapping.iteritems():
+        if (k[0] is None or k[0] in mc) and k[1] == p:
+          if isinstance(o, model.core.AbstractObject): v = "<%s>" % str(o.uri)
+          else:
+            if m.to_rdf is not None: o = m.to_rdf(o)
+            v = '"%s"' % str(o)
+            if m.datatype is not None: v += "^^<%s>" % str(m.datatype)
+          self._sparqlstore.update_triples(graph_uri,
+            [("<%s>" % str(s.uri), "<%s>" % str(m.property), v)])
+          return
