@@ -58,7 +58,6 @@ class StreamClient(ws4py.client.threadedclient.WebSocketClient):
     self._parser = stream.BlockParser(self._receiver, check=check)
     self._opened = False
     self._access_key = access_key
-    self._req_close = False
 
   @property
   def handshake_headers(self):
@@ -78,14 +77,10 @@ class StreamClient(ws4py.client.threadedclient.WebSocketClient):
 
   def close(self, *args):
   #----------------------
-    self._req_close = True
     super(StreamClient, self).close(*args)
 
   def closed(self, code, reason=None):
   #-----------------------------------
-    if not self._req_close:
-      pass   ### We need to cleanly raise an exception and stop thread,,,
-      ## logging.debug('UNREQUESTED CLOSE...')
     self._receiver(None)
 
   def received_message(self, msg):
@@ -140,9 +135,10 @@ class WebStreamReader(stream.SignalDataStream):
   ..todo:: Document extra parameters...
   """
   def __init__(self, endpoint, uri,
-    start=None, offset=None, duration=-1, count=None, maxsize=-1, dtype=None, units=None, **kwds):
-  #-----------------------------------------------------------------------------------------------
-    super(WebStreamReader, self).__init__(endpoint, uri, start, offset, duration, count, maxsize, dtype, units)
+    start=None, offset=None, duration=-1, count=None, maxsize=-1, dtype=None, rate=None, units=None, **kwds):
+  #---------------------------------------------------------------------------------------------------------
+    super(WebStreamReader, self).__init__(endpoint, uri, start, offset, duration, count, maxsize,
+                                                                                         dtype, rate, units)
     try:
       self._ws = StreamClient(endpoint, self._request, self._receiveQ, protocols=['biosignalml-ssf'], **kwds)
       self._ws.connect()
@@ -165,7 +161,8 @@ class WebStreamWriter(object):
   def __init__(self, endpoint, access_key=None):
   #---------------------------------------------
     try:
-      self._ws = StreamClient(endpoint, None, self.got_response, protocols=['biosignalml-ssf'])
+      self._ws = StreamClient(endpoint, None, self.got_response, access_key=access_key,
+                              protocols=['biosignalml-ssf'])
       self._ws.connect()
     except Exception, msg:
       logging.error('Unable to connect to WebSocket: %s', msg)
@@ -187,6 +184,7 @@ class WebStreamWriter(object):
   def got_response(block):
   #-----------------------
     if block and block.type == stream.BlockType.ERROR:
+      logging.error("STREAM ERROR: %s:", block.content)
       raise stream.StreamException(block.content)
 
 
