@@ -18,13 +18,16 @@
 #
 ######################################################
 
+import logging
+
 from biosignalml.rdf import Resource, NS
 
 from ontology import UNITS
 
 
 _direct = {
-  'bmp':    'BeatsPerMinute',
+  'Bpm':    'BeatsPerMinute',
+  'bpm':    'BeatsPerMinute',
   'cc':     'CubicCentimetre',
   'pm':     'PerMinute',
   '1/min':  'PerMinute',
@@ -115,14 +118,16 @@ def get_units_uri(s):
   """
   if s:
     try:
-      uome = _direct[s]
+      unit = _direct[s]
     except KeyError:
-      try: uome = 'Per'.join([_mult(u) for u in s.split('/')])
+      try: unit = 'Per'.join([_mult(u) for u in s.split('/')])
       except KeyError:
-        raise ValueError("Invalid units abbreviation: %s" % s)
-    resource = getattr(UNITS, uome, None)
+        if s.startswith('per_'): unit = 'Per' + _upperfirst(s[4:])
+        else:                    unit = _upperfirst(s)
+    resource = getattr(UNITS, unit, None)
     if resource is not None: return resource.uri
-  raise ValueError("Unknown units abbreviation: %s" % s)
+    raise ValueError("Unknown units abbreviation: %s" % s)
+
 
 if __name__ == '__main__':
 #=========================
