@@ -19,7 +19,6 @@
 ######################################################
 
 import pint
-import logging
 
 import biosignalml.rdf             as rdf
 import biosignalml.rdf.sparqlstore as sparqlstore
@@ -27,6 +26,7 @@ import biosignalml.rdf.sparqlstore as sparqlstore
 from biosignalml.rdf import RDF
 UOME_CORE = rdf.NS('http://www.sbpax.org/uome/core.owl#')
 
+'''Graph holding http://www.sbpax.org/uome/core.owl#UnitOfMeasurement resources.'''
 UNITS_GRAPH = "http://ontologies.biosignalml.org/units"
 
 UNIT_PREFIXES = [ rdf.UOME.prefix,
@@ -114,7 +114,6 @@ class UnitStore(object):
   def get_unit(self, uri):
   #-----------------------
     uri = str(uri)
-    #logging.debug('Looking for %s', uri)
     unit = self._cache.get(uri)
     if unit is None:
       name = strip_prefix(uri)
@@ -146,7 +145,7 @@ class UnitStore(object):
     return unit
 
 
-class UnitConvertor(object):
+class UnitConverter(object):
 #===========================
 
   def __init__(self, store):
@@ -158,17 +157,20 @@ class UnitConvertor(object):
     """
     Convert between compatible units.
 
-    :return: A function mapping values in `drom_units` to `to_units`.
+    :return: A function mapping values in `from_units` to `to_units`.
     """
     ratio = self._store.get_unit(from_units)/self._store.get_unit(to_units)
-    if ratio.unitless: return lambda x: x*ratio.magnitude + 0.0   #: (scale, offset)
-    else: raise TypeError("Cannot convert between %s and %s" % (from_units, to_units))
+    if ratio.unitless:
+      if ratio.magnitude == 1.0: return lambda x: x
+      else:                      return lambda x: x*ratio.magnitude + 0.0   #: (scale, offset)
+    else:
+      raise TypeError("Cannot convert between %s and %s" % (from_units, to_units))
 
 
 if __name__ == '__main__':
 #=========================
 
-  store = UnitConvertor(sparqlstore.Virtuoso('http://localhost:8890'))
+  store = UnitConverter(sparqlstore.Virtuoso('http://localhost:8890'))
 
   def test(u):
   #-----------
